<?php

namespace Planaday\Database\Abstraction\SQLQuery;

interface ColumnTypeMap {
	public function UUID(): string;
	public function HASH(): string;
	public function LAST_UPDATED_TIMESTAMP(): string;
	public function CREATION_TIMESTAMP(): string;
	public function PRICE(): string;
	public function TAX(): string;
	public function DATE(): string;
	public function DATETIME(): string;
	public function TIME(): string;
	public function TEXT(): string;
	public function BOOLEAN(): string;
	public function ID(): string;
	public function STRING_ID(): string;
	public function NAME(): string;
	public function STATUS(): string;
	public function DESCRIPTION(): string;
	public function REMARK(): string;
	public function AVAILABILITY_COUNT(): string;
	public function CODE(): string;
	public function LABELS(): string;
	public function STREET(): string;
	public function HOUSE_NUMBER(): string;
	public function HOUSE_NUMBER_EXTENSION(): string;
	public function ZIPCODE(): string;
	public function CITY(): string;
	public function COUNTRY(): string;
	public function COORDINATES(): string;
	public function PHONE_NUMBER(): string;
	public function EMAIL(): string;
	public function WEBSITE(): string;


	public function AUTO_INCREMENT(): string;
	public function PRIMARY_KEY(): string;
	public function FOREIGN_KEY(): string;
	public function NOT_NULL(): string;
	public function DEFAULT_NULL(): string;
}